const util = require("util");
const parse = require("parse-aws-lambda-name");
const assert = require("assert");
const Boom = require("boom");
const Package = require("../../package.json");

class LambdaRepository {
  constructor({ dataService, loggerService }) {
    assert(loggerService, "Logger is required");
    assert(dataService, "Data Service is required");
    this.logger_ = loggerService;
    this.dataService_ = dataService;
  }

  async invoke({ name, invokePayload }) {
    try {
      assert(name, "Name is required");
      const parsed = parse(name);
      if (!parsed) {
        throw Boom.badRequest("Lambda function name is invalid");
      }
      const params = {
        "FunctionName": parsed.functionName,
        "InvocationType": "RequestResponse",
        "LogType": "Tail",
        "ClientContext": this.dataService_.aws_util.base64.encode(JSON.stringify(Package.name))
      };
      if (invokePayload) {
        params.Payload = JSON.stringify(invokePayload);
      }
      if (parsed.qualifier) {
        params.Qualifier = parsed.qualifier;
      }
      const data = await util.promisify(this.dataService_.invoke.bind(this.dataService_))(params);
      let payload = data.Payload;
      payload = JSON.parse(payload);
      if (payload && payload.errorMessage) {
        throw Boom.badImplementation(payload.errorMessage);
      }
      return payload;
    } catch (error) {
      this.errorHandler_(error);
    }
  }

  async updateLambdaConfig(config) {

    let params;
    const promiseArr = [];
    try {

      assert(config, "Name is required");
      if (!config) {
        throw Boom.badRequest("Lambda function name is invalid");
      }

      if (Array.isArray(config.functions)) {
        config.functions.forEach(con => {

          if (con.name) {
            params = {
              "FunctionName": con.name,
              "Environment": {
                "Variables": con.envVariables
              }
            };
            promiseArr.push(util.promisify(this.dataService_.updateFunctionConfiguration.bind(this.dataService_))(params));
          }
        });

      } else {
        params = {
          "FunctionName": config.name,
          "Environment": {
            "Variables": config.envVariables
          }
        };
        promiseArr.push(util.promisify(this.dataService_.updateFunctionConfiguration.bind(this.dataService_))(params));
      }

      const data = await Promise.all(promiseArr);

      return data;
    } catch (error) {
      this.errorHandler_(error);
    }

  }

  async invokeAsync({name, invokePayload}) {
    try {
      assert(name, "Name is required");
      const parsed = parse(name);
      if (!parsed) {
        throw Boom.badRequest("Lambda function name is invalid");
      }
      const params = {
        "FunctionName": parsed.functionName,
        "InvocationType": "Event",
        "LogType": "Tail",
        "ClientContext": this.dataService_.aws_util.base64.encode(JSON.stringify(Package.name))
      };
      if (invokePayload) {
        params.Payload = JSON.stringify(invokePayload);
      }
      if (parsed.qualifier) {
        params.Qualifier = parsed.qualifier;
      }
      return util.promisify(this.dataService_.invoke.bind(this.dataService_))(params);
    } catch (error) {
      this.errorHandler_(error);
    }

  }

  errorHandler_(error) {
    this.logger_.error("An error occurred in lambda service", error);
    throw error;
  }

}

module.exports = LambdaRepository;
